@@ -1,28 +1,67 @@
-use std::fmt::Debug;
-use std::time::Duration;
-use std::{borrow::Cow, collections::VecDeque};
+use std::{
+    borrow::Cow,
+    collections::VecDeque,
+    fmt::{Debug, Display, Formatter},
+    time::Duration,
+};
 
 use humantime::parse_duration;
+use regex::Regex;
 
-use super::data_harvester::processes::ProcessHarvest;
-use crate::utils::error::{
-    BottomError::{self, QueryError},
-    Result,
+use crate::{
+    data_collection::processes::ProcessHarvest, multi_eq_ignore_ascii_case, utils::data_prefixes::*,
 };
 
+#[derive(Debug)]
+pub(crate) struct QueryError {
+    reason: Cow<'static, str>,
+}
+
+impl QueryError {
+    #[inline]
+    pub(crate) fn new<I: Into<Cow<'static, str>>>(reason: I) -> Self {
+        Self {
+            reason: reason.into(),
+        }
+    }
+
+    #[inline]
+    fn missing_value() -> Self {
+        Self {
+            reason: "Missing value".into(),
+        }
+    }
+}
+
+impl Display for QueryError {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        write!(f, "{}", self.reason)
+    }
+}
+
+impl From<regex::Error> for QueryError {
+    fn from(err: regex::Error) -> Self {
+        Self::new(err.to_string())
+    }
+}
+
+type QueryResult<T> = Result<T, QueryError>;
+
 const DELIMITER_LIST: [char; 6] = ['=', '>', '<', '(', ')', '\"'];
 const COMPARISON_LIST: [&str; 3] = [">", "=", "<"];
 const OR_LIST: [&str; 2] = ["or", "||"];
 const AND_LIST: [&str; 2] = ["and", "&&"];
 
 /// In charge of parsing the given query.
-/// We are defining the following language for a query (case-insensitive prefixes):
+/// We are defining the following language for a query (case-insensitive
+/// prefixes):
 ///
 /// - Process names: No prefix required, can use regex, match word, or case.
-///   Enclosing anything, including prefixes, in quotes, means we treat it as an entire process
-///   rather than a prefix.
+///   Enclosing anything, including prefixes, in quotes, means we treat it as an
+///   entire process rather than a prefix.
 /// - PIDs: Use prefix `pid`, can use regex or match word (case is irrelevant).
-/// - CPU: Use prefix `cpu`, cannot use r/m/c (regex, match word, case).  Can compare.
+/// - CPU: Use prefix `cpu`, cannot use r/m/c (regex, match word, case).  Can
+///   compare.
 /// - MEM: Use prefix `mem`, cannot use r/m/c.  Can compare.
 /// - STATE: Use prefix `state`, can use regex, match word, or case.
 /// - USER: Use prefix `user`, can use regex, match word, or case.
@@ -31,14 +70,15 @@ const AND_LIST: [&str; 2] = ["and", "&&"];
 /// - Total read: Use prefix `read`.  Can compare.
 /// - Total write: Use prefix `write`.  Can compare.
 ///
-/// For queries, whitespaces are our delimiters.  We will merge together any adjacent non-prefixed
-/// or quoted elements after splitting to treat as process names.
-/// Furthermore, we want to support boolean joiners like AND and OR, and brackets.
-pub fn parse_query(
+/// For queries, whitespaces are our delimiters.  We will merge together any
+/// adjacent non-prefixed or quoted elements after splitting to treat as process
+/// names. Furthermore, we want to support boolean joiners like AND and OR, and
+/// brackets.
+pub(crate) fn parse_query(
     search_query: &str, is_searching_whole_word: bool, is_ignoring_case: bool,
     is_searching_with_regex: bool,
-) -> Result<Query> {
-    fn process_string_to_filter(query: &mut VecDeque<String>) -> Result<Query> {
+) -> QueryResult<Query> {
+    fn process_string_to_filter(query: &mut VecDeque<String>) -> QueryResult<Query> {
         let lhs = process_or(query)?;
         let mut list_of_ors = vec![lhs];
 
@@ -49,13 +89,13 @@ pub fn parse_query(
         Ok(Query { query: list_of_ors })
     }
 
-    fn process_or(query: &mut VecDeque<String>) -> Result<Or> {
+    fn process_or(query: &mut VecDeque<String>) -> QueryResult<Or> {
         let mut lhs = process_and(query)?;
         let mut rhs: Option<Box<And>> = None;
 
         while let Some(queue_top) = query.front() {
-            // debug!("OR QT: {:?}", queue_top);
-            if OR_LIST.contains(&queue_top.to_lowercase().as_str()) {
+            let current_lowercase = queue_top.to_lowercase();
+            if OR_LIST.contains(&current_lowercase.as_str()) {
                 query.pop_front();
                 rhs = Some(Box::new(process_and(query)?));
 
@@ -75,8 +115,8 @@ pub fn parse_query(
                 } else {
                     break;
                 }
-            } else if COMPARISON_LIST.contains(&queue_top.to_lowercase().as_str()) {
-                return Err(QueryError(Cow::Borrowed("Comparison not valid here")));
+            } else if COMPARISON_LIST.contains(&current_lowercase.as_str()) {
+                return Err(QueryError::new("Comparison not valid here"));
             } else {
                 break;
             }
@@ -85,13 +125,13 @@ pub fn parse_query(
         Ok(Or { lhs, rhs })
     }
 
-    fn process_and(query: &mut VecDeque<String>) -> Result<And> {
+    fn process_and(query: &mut VecDeque<String>) -> QueryResult<And> {
         let mut lhs = process_prefix(query, false)?;
         let mut rhs: Option<Box<Prefix>> = None;
 
         while let Some(queue_top) = query.front() {
-            // debug!("AND QT: {:?}", queue_top);
-            if AND_LIST.contains(&queue_top.to_lowercase().as_str()) {
+            let current_lowercase = queue_top.to_lowercase();
+            if AND_LIST.contains(&current_lowercase.as_str()) {
                 query.pop_front();
 
                 rhs = Some(Box::new(process_prefix(query, false)?));
@@ -114,8 +154,8 @@ pub fn parse_query(
                 } else {
                     break;
                 }
-            } else if COMPARISON_LIST.contains(&queue_top.to_lowercase().as_str()) {
-                return Err(QueryError(Cow::Borrowed("Comparison not valid here")));
+            } else if COMPARISON_LIST.contains(&current_lowercase.as_str()) {
+                return Err(QueryError::new("Comparison not valid here"));
             } else {
                 break;
             }
@@ -124,12 +164,51 @@ pub fn parse_query(
         Ok(And { lhs, rhs })
     }
 
-    fn process_prefix(query: &mut VecDeque<String>, inside_quotation: bool) -> Result<Prefix> {
+    #[inline]
+    fn process_prefix_units(query: &mut VecDeque<String>, value: &mut f64) {
+        // If no unit, assume base.
+        //
+        // Furthermore, base must be PEEKED at initially, and will
+        // require (likely) prefix_type specific checks
+        // Lastly, if it *is* a unit, remember to POP!
+        if let Some(potential_unit) = query.front() {
+            if potential_unit.eq_ignore_ascii_case("tb") {
+                *value *= TERA_LIMIT_F64;
+                query.pop_front();
+            } else if potential_unit.eq_ignore_ascii_case("tib") {
+                *value *= TEBI_LIMIT_F64;
+                query.pop_front();
+            } else if potential_unit.eq_ignore_ascii_case("gb") {
+                *value *= GIGA_LIMIT_F64;
+                query.pop_front();
+            } else if potential_unit.eq_ignore_ascii_case("gib") {
+                *value *= GIBI_LIMIT_F64;
+                query.pop_front();
+            } else if potential_unit.eq_ignore_ascii_case("mb") {
+                *value *= MEGA_LIMIT_F64;
+                query.pop_front();
+            } else if potential_unit.eq_ignore_ascii_case("mib") {
+                *value *= MEBI_LIMIT_F64;
+                query.pop_front();
+            } else if potential_unit.eq_ignore_ascii_case("kb") {
+                *value *= KILO_LIMIT_F64;
+                query.pop_front();
+            } else if potential_unit.eq_ignore_ascii_case("kib") {
+                *value *= KIBI_LIMIT_F64;
+                query.pop_front();
+            } else if potential_unit.eq_ignore_ascii_case("b") {
+                query.pop_front();
+            }
+        }
+    }
+
+    fn process_prefix(query: &mut VecDeque<String>, inside_quotation: bool) -> QueryResult<Prefix> {
         if let Some(queue_top) = query.pop_front() {
             if inside_quotation {
                 if queue_top == "\"" {
-                    // This means we hit something like "".  Return an empty prefix, and to deal with
-                    // the close quote checker, add one to the top of the stack.  Ugly fix but whatever.
+                    // This means we hit something like "".  Return an empty prefix, and to deal
+                    // with the close quote checker, add one to the top of the
+                    // stack.  Ugly fix but whatever.
                     query.push_front("\"".to_string());
                     return Ok(Prefix {
                         or: None,
@@ -158,7 +237,7 @@ pub fn parse_query(
                 }
             } else if queue_top == "(" {
                 if query.is_empty() {
-                    return Err(QueryError(Cow::Borrowed("Missing closing parentheses")));
+                    return Err(QueryError::new("Missing closing parentheses"));
                 }
 
                 let mut list_of_ors = VecDeque::new();
@@ -173,7 +252,7 @@ pub fn parse_query(
 
                 // Ensure not empty
                 if list_of_ors.is_empty() {
-                    return Err(QueryError("No values within parentheses group".into()));
+                    return Err(QueryError::new("No values within parentheses group"));
                 }
 
                 // Now convert this back to a OR...
@@ -212,29 +291,30 @@ pub fn parse_query(
                             compare_prefix: None,
                         });
                     } else {
-                        return Err(QueryError("Missing closing parentheses".into()));
+                        return Err(QueryError::new("Missing closing parentheses"));
                     }
                 } else {
-                    return Err(QueryError("Missing closing parentheses".into()));
+                    return Err(QueryError::new("Missing closing parentheses"));
                 }
             } else if queue_top == ")" {
-                return Err(QueryError("Missing opening parentheses".into()));
+                return Err(QueryError::new("Missing opening parentheses"));
             } else if queue_top == "\"" {
-                // Similar to parentheses, trap and check for missing closing quotes.  Note, however, that we
-                // will DIRECTLY call another process_prefix call...
+                // Similar to parentheses, trap and check for missing closing quotes.  Note,
+                // however, that we will DIRECTLY call another process_prefix
+                // call...
 
                 let prefix = process_prefix(query, true)?;
                 if let Some(close_paren) = query.pop_front() {
                     if close_paren == "\"" {
                         return Ok(prefix);
                     } else {
-                        return Err(QueryError("Missing closing quotation".into()));
+                        return Err(QueryError::new("Missing closing quotation"));
                     }
                 } else {
-                    return Err(QueryError("Missing closing quotation".into()));
+                    return Err(QueryError::new("Missing closing quotation"));
                 }
             } else {
-                //  Get prefix type...
+                // Get prefix type.
                 let prefix_type = queue_top.parse::<PrefixType>()?;
                 let content = if let PrefixType::Name = prefix_type {
                     Some(queue_top)
@@ -260,10 +340,12 @@ pub fn parse_query(
                                     // - (test)
                                     // - (test
                                     // - test)
-                                    // These are split into 2 to 3 different strings due to parentheses being
+                                    // These are split into 2 to 3 different strings due to
+                                    // parentheses being
                                     // delimiters in our query system.
                                     //
-                                    // Do we want these to be valid?  They should, as a string, right?
+                                    // Do we want these to be valid?  They should, as a string,
+                                    // right?
 
                                     return Ok(Prefix {
                                         or: None,
@@ -307,15 +389,15 @@ pub fn parse_query(
                                         duration_string = Some(queue_next);
                                     }
                                 } else {
-                                    return Err(QueryError("Missing value".into()));
+                                    return Err(QueryError::missing_value());
                                 }
                             }
 
                             if let Some(condition) = condition {
                                 let duration = parse_duration(
-                                    &duration_string.ok_or(QueryError("Missing value".into()))?,
+                                    &duration_string.ok_or(QueryError::missing_value())?,
                                 )
-                                .map_err(|err| QueryError(err.to_string().into()))?;
+                                .map_err(|err| QueryError::new(err.to_string()))?;
 
                                 return Ok(Prefix {
                                     or: None,
@@ -328,7 +410,6 @@ pub fn parse_query(
                                         }),
                                     )),
                                 });
-                            } else {
                             }
                         }
                         _ => {
@@ -338,14 +419,14 @@ pub fn parse_query(
                             let mut condition: Option<QueryComparison> = None;
                             let mut value: Option<f64> = None;
 
-                            // TODO: Jeez, what the heck did I write here... add some tests and clean this up in the
-                            // future.
+                            // TODO: Jeez, what the heck did I write here... add some tests and
+                            // clean this up in the future.
                             if content == "=" {
                                 condition = Some(QueryComparison::Equal);
                                 if let Some(queue_next) = query.pop_front() {
                                     value = queue_next.parse::<f64>().ok();
                                 } else {
-                                    return Err(QueryError("Missing value".into()));
+                                    return Err(QueryError::missing_value());
                                 }
                             } else if content == ">" || content == "<" {
                                 // We also have to check if the next string is an "="...
@@ -359,7 +440,7 @@ pub fn parse_query(
                                         if let Some(queue_next_next) = query.pop_front() {
                                             value = queue_next_next.parse::<f64>().ok();
                                         } else {
-                                            return Err(QueryError("Missing value".into()));
+                                            return Err(QueryError::missing_value());
                                         }
                                     } else {
                                         condition = Some(if content == ">" {
@@ -370,14 +451,15 @@ pub fn parse_query(
                                         value = queue_next.parse::<f64>().ok();
                                     }
                                 } else {
-                                    return Err(QueryError("Missing value".into()));
+                                    return Err(QueryError::missing_value());
                                 }
                             }
 
                             if let Some(condition) = condition {
                                 if let Some(read_value) = value {
-                                    // Note that the values *might* have a unit or need to be parsed differently
-                                    // based on the prefix type!
+                                    // Note that the values *might* have a unit or need to be parsed
+                                    // differently based on the
+                                    // prefix type!
 
                                     let mut value = read_value;
 
@@ -387,52 +469,11 @@ pub fn parse_query(
                                         | PrefixType::Wps
                                         | PrefixType::TRead
                                         | PrefixType::TWrite => {
-                                            // If no unit, assume base.
-                                            // Furthermore, base must be PEEKED at initially, and will
-                                            // require (likely) prefix_type specific checks
-                                            // Lastly, if it *is* a unit, remember to POP!
-
-                                            if let Some(potential_unit) = query.front() {
-                                                match potential_unit.to_lowercase().as_str() {
-                                                    "tb" => {
-                                                        value *= 1_000_000_000_000.0;
-                                                        query.pop_front();
-                                                    }
-                                                    "tib" => {
-                                                        value *= 1_099_511_627_776.0;
-                                                        query.pop_front();
-                                                    }
-                                                    "gb" => {
-                                                        value *= 1_000_000_000.0;
-                                                        query.pop_front();
-                                                    }
-                                                    "gib" => {
-                                                        value *= 1_073_741_824.0;
-                                                        query.pop_front();
-                                                    }
-                                                    "mb" => {
-                                                        value *= 1_000_000.0;
-                                                        query.pop_front();
-                                                    }
-                                                    "mib" => {
-                                                        value *= 1_048_576.0;
-                                                        query.pop_front();
-                                                    }
-                                                    "kb" => {
-                                                        value *= 1000.0;
-                                                        query.pop_front();
-                                                    }
-                                                    "kib" => {
-                                                        value *= 1024.0;
-                                                        query.pop_front();
-                                                    }
-                                                    "b" => {
-                                                        // Just gotta pop.
-                                                        query.pop_front();
-                                                    }
-                                                    _ => {}
-                                                }
-                                            }
+                                            process_prefix_units(query, &mut value);
+                                        }
+                                        #[cfg(feature = "gpu")]
+                                        PrefixType::GMem => {
+                                            process_prefix_units(query, &mut value);
                                         }
                                         _ => {}
                                     }
@@ -453,21 +494,21 @@ pub fn parse_query(
                         }
                     }
                 } else {
-                    return Err(QueryError("Missing argument for search prefix".into()));
+                    return Err(QueryError::new("Missing argument for search prefix"));
                 }
             }
         } else if inside_quotation {
             // Uh oh, it's empty with quotes!
-            return Err(QueryError("Missing closing quotation".into()));
+            return Err(QueryError::new("Missing closing quotation"));
         }
 
-        Err(QueryError("Invalid query".into()))
+        Err(QueryError::new("Invalid query"))
     }
 
     let mut split_query = VecDeque::new();
 
     search_query.split_whitespace().for_each(|s| {
-        // From https://stackoverflow.com/a/56923739 in order to get a split but include the parentheses
+        // From https://stackoverflow.com/a/56923739 in order to get a split, but include the parentheses
         let mut last = 0;
         for (index, matched) in s.match_indices(|x| DELIMITER_LIST.contains(&x)) {
             if last != index {
@@ -493,14 +534,14 @@ pub fn parse_query(
 
 pub struct Query {
     /// Remember, AND > OR, but AND must come after OR when we parse.
-    pub query: Vec<Or>,
+    query: Vec<Or>,
 }
 
 impl Query {
-    pub fn process_regexes(
+    fn process_regexes(
         &mut self, is_searching_whole_word: bool, is_ignoring_case: bool,
         is_searching_with_regex: bool,
-    ) -> Result<()> {
+    ) -> QueryResult<()> {
         for or in &mut self.query {
             or.process_regexes(
                 is_searching_whole_word,
@@ -512,7 +553,7 @@ impl Query {
         Ok(())
     }
 
-    pub fn check(&self, process: &ProcessHarvest, is_using_command: bool) -> bool {
+    pub(crate) fn check(&self, process: &ProcessHarvest, is_using_command: bool) -> bool {
         self.query
             .iter()
             .all(|ok| ok.check(process, is_using_command))
@@ -520,22 +561,22 @@ impl Query {
 }
 
 impl Debug for Query {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
         f.write_fmt(format_args!("{:?}", self.query))
     }
 }
 
 #[derive(Default)]
-pub struct Or {
-    pub lhs: And,
-    pub rhs: Option<Box<And>>,
+struct Or {
+    lhs: And,
+    rhs: Option<Box<And>>,
 }
 
 impl Or {
-    pub fn process_regexes(
+    fn process_regexes(
         &mut self, is_searching_whole_word: bool, is_ignoring_case: bool,
         is_searching_with_regex: bool,
-    ) -> Result<()> {
+    ) -> QueryResult<()> {
         self.lhs.process_regexes(
             is_searching_whole_word,
             is_ignoring_case,
@@ -552,7 +593,7 @@ impl Or {
         Ok(())
     }
 
-    pub fn check(&self, process: &ProcessHarvest, is_using_command: bool) -> bool {
+    fn check(&self, process: &ProcessHarvest, is_using_command: bool) -> bool {
         if let Some(rhs) = &self.rhs {
             self.lhs.check(process, is_using_command) || rhs.check(process, is_using_command)
         } else {
@@ -562,7 +603,7 @@ impl Or {
 }
 
 impl Debug for Or {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
         match &self.rhs {
             Some(rhs) => f.write_fmt(format_args!("({:?} OR {:?})", self.lhs, rhs)),
             None => f.write_fmt(format_args!("{:?}", self.lhs)),
@@ -571,16 +612,16 @@ impl Debug for Or {
 }
 
 #[derive(Default)]
-pub struct And {
-    pub lhs: Prefix,
-    pub rhs: Option<Box<Prefix>>,
+struct And {
+    lhs: Prefix,
+    rhs: Option<Box<Prefix>>,
 }
 
 impl And {
-    pub fn process_regexes(
+    fn process_regexes(
         &mut self, is_searching_whole_word: bool, is_ignoring_case: bool,
         is_searching_with_regex: bool,
-    ) -> Result<()> {
+    ) -> QueryResult<()> {
         self.lhs.process_regexes(
             is_searching_whole_word,
             is_ignoring_case,
@@ -597,7 +638,7 @@ impl And {
         Ok(())
     }
 
-    pub fn check(&self, process: &ProcessHarvest, is_using_command: bool) -> bool {
+    fn check(&self, process: &ProcessHarvest, is_using_command: bool) -> bool {
         if let Some(rhs) = &self.rhs {
             self.lhs.check(process, is_using_command) && rhs.check(process, is_using_command)
         } else {
@@ -607,7 +648,7 @@ impl And {
 }
 
 impl Debug for And {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
         match &self.rhs {
             Some(rhs) => f.write_fmt(format_args!("({:?} AND {:?})", self.lhs, rhs)),
             None => f.write_fmt(format_args!("{:?}", self.lhs)),
@@ -616,7 +657,7 @@ impl Debug for And {
 }
 
 #[derive(Debug)]
-pub enum PrefixType {
+enum PrefixType {
     Pid,
     PCpu,
     MemBytes,
@@ -629,88 +670,114 @@ pub enum PrefixType {
     State,
     User,
     Time,
+    #[cfg(feature = "gpu")]
+    PGpu,
+    #[cfg(feature = "gpu")]
+    GMem,
+    #[cfg(feature = "gpu")]
+    PGMem,
     __Nonexhaustive,
 }
 
 impl std::str::FromStr for PrefixType {
-    type Err = BottomError;
+    type Err = QueryError;
 
-    fn from_str(s: &str) -> Result<Self> {
+    fn from_str(s: &str) -> QueryResult<Self> {
         use PrefixType::*;
 
-        let lower_case = s.to_lowercase();
-        // Didn't add mem_bytes, total_read, and total_write
+        // TODO: Didn't add mem_bytes, total_read, and total_write
         // for now as it causes help to be clogged.
-        match lower_case.as_str() {
-            "cpu" | "cpu%" => Ok(PCpu),
-            "mem" | "mem%" => Ok(PMem),
-            "memb" => Ok(MemBytes),
-            "read" | "r/s" | "rps" => Ok(Rps),
-            "write" | "w/s" | "wps" => Ok(Wps),
-            "tread" | "t.read" => Ok(TRead),
-            "twrite" | "t.write" => Ok(TWrite),
-            "pid" => Ok(Pid),
-            "state" => Ok(State),
-            "user" => Ok(User),
-            "time" => Ok(Time),
-            _ => Ok(Name),
+
+        let mut result = Name;
+        if multi_eq_ignore_ascii_case!(s, "cpu" | "cpu%") {
+            result = PCpu;
+        } else if multi_eq_ignore_ascii_case!(s, "mem" | "mem%") {
+            result = PMem;
+        } else if multi_eq_ignore_ascii_case!(s, "memb") {
+            result = MemBytes;
+        } else if multi_eq_ignore_ascii_case!(s, "read" | "r/s" | "rps") {
+            result = Rps;
+        } else if multi_eq_ignore_ascii_case!(s, "write" | "w/s" | "wps") {
+            result = Wps;
+        } else if multi_eq_ignore_ascii_case!(s, "tread" | "t.read") {
+            result = TRead;
+        } else if multi_eq_ignore_ascii_case!(s, "twrite" | "t.write") {
+            result = TWrite;
+        } else if multi_eq_ignore_ascii_case!(s, "pid") {
+            result = Pid;
+        } else if multi_eq_ignore_ascii_case!(s, "state") {
+            result = State;
+        } else if multi_eq_ignore_ascii_case!(s, "user") {
+            result = User;
+        } else if multi_eq_ignore_ascii_case!(s, "time") {
+            result = Time;
         }
+        #[cfg(feature = "gpu")]
+        {
+            if multi_eq_ignore_ascii_case!(s, "gmem") {
+                result = GMem;
+            } else if multi_eq_ignore_ascii_case!(s, "gmem%") {
+                result = PGMem;
+            } else if multi_eq_ignore_ascii_case!(s, "gpu%") {
+                result = PGpu;
+            }
+        }
+        Ok(result)
     }
 }
 
-// TODO: This is also jank and could be better represented. Add tests, then clean up!
+// TODO: This is also jank and could be better represented. Add tests, then
+// clean up!
 #[derive(Default)]
-pub struct Prefix {
-    pub or: Option<Box<Or>>,
-    pub regex_prefix: Option<(PrefixType, StringQuery)>,
-    pub compare_prefix: Option<(PrefixType, ComparableQuery)>,
+struct Prefix {
+    or: Option<Box<Or>>,
+    regex_prefix: Option<(PrefixType, StringQuery)>,
+    compare_prefix: Option<(PrefixType, ComparableQuery)>,
 }
 
 impl Prefix {
-    pub fn process_regexes(
+    fn process_regexes(
         &mut self, is_searching_whole_word: bool, is_ignoring_case: bool,
         is_searching_with_regex: bool,
-    ) -> Result<()> {
+    ) -> QueryResult<()> {
         if let Some(or) = &mut self.or {
             return or.process_regexes(
                 is_searching_whole_word,
                 is_ignoring_case,
                 is_searching_with_regex,
             );
-        } else if let Some((prefix_type, StringQuery::Value(regex_string))) = &mut self.regex_prefix
+        } else if let Some((
+            PrefixType::Pid | PrefixType::Name | PrefixType::State | PrefixType::User,
+            StringQuery::Value(regex_string),
+        )) = &mut self.regex_prefix
         {
-            match prefix_type {
-                PrefixType::Pid | PrefixType::Name | PrefixType::State | PrefixType::User => {
-                    let escaped_regex: String;
-                    let final_regex_string = &format!(
-                        "{}{}{}{}",
-                        if is_searching_whole_word { "^" } else { "" },
-                        if is_ignoring_case { "(?i)" } else { "" },
-                        if !is_searching_with_regex {
-                            escaped_regex = regex::escape(regex_string);
-                            &escaped_regex
-                        } else {
-                            regex_string
-                        },
-                        if is_searching_whole_word { "$" } else { "" },
-                    );
+            let escaped_regex: String;
+            let final_regex_string = &format!(
+                "{}{}{}{}",
+                if is_searching_whole_word { "^" } else { "" },
+                if is_ignoring_case { "(?i)" } else { "" },
+                if !is_searching_with_regex {
+                    escaped_regex = regex::escape(regex_string);
+                    &escaped_regex
+                } else {
+                    regex_string
+                },
+                if is_searching_whole_word { "$" } else { "" },
+            );
 
-                    let taken_pwc = self.regex_prefix.take();
-                    if let Some((taken_pt, _)) = taken_pwc {
-                        self.regex_prefix = Some((
-                            taken_pt,
-                            StringQuery::Regex(regex::Regex::new(final_regex_string)?),
-                        ));
-                    }
-                }
-                _ => {}
+            let taken_pwc = self.regex_prefix.take();
+            if let Some((taken_pt, _)) = taken_pwc {
+                self.regex_prefix = Some((
+                    taken_pt,
+                    StringQuery::Regex(Regex::new(final_regex_string)?),
+                ));
             }
         }
 
         Ok(())
     }
 
-    pub fn check(&self, process: &ProcessHarvest, is_using_command: bool) -> bool {
+    fn check(&self, process: &ProcessHarvest, is_using_command: bool) -> bool {
         fn matches_condition<I: Into<f64>, J: Into<f64>>(
             condition: &QueryComparison, lhs: I, rhs: J,
         ) -> bool {
@@ -718,7 +785,7 @@ impl Prefix {
             let rhs: f64 = rhs.into();
 
             match condition {
-                QueryComparison::Equal => (lhs - rhs).abs() < std::f64::EPSILON,
+                QueryComparison::Equal => (lhs - rhs).abs() < f64::EPSILON,
                 QueryComparison::Less => lhs < rhs,
                 QueryComparison::Greater => lhs > rhs,
                 QueryComparison::LessOrEqual => lhs <= rhs,
@@ -792,6 +859,24 @@ impl Prefix {
                         process.total_write_bytes as f64,
                         numerical_query.value,
                     ),
+                    #[cfg(feature = "gpu")]
+                    PrefixType::PGpu => matches_condition(
+                        &numerical_query.condition,
+                        process.gpu_util,
+                        numerical_query.value,
+                    ),
+                    #[cfg(feature = "gpu")]
+                    PrefixType::GMem => matches_condition(
+                        &numerical_query.condition,
+                        process.gpu_mem as f64,
+                        numerical_query.value,
+                    ),
+                    #[cfg(feature = "gpu")]
+                    PrefixType::PGMem => matches_condition(
+                        &numerical_query.condition,
+                        process.gpu_mem_percent,
+                        numerical_query.value,
+                    ),
                     _ => true,
                 },
                 ComparableQuery::Time(time_query) => match prefix_type {
@@ -809,21 +894,21 @@ impl Prefix {
 }
 
 impl Debug for Prefix {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
         if let Some(or) = &self.or {
-            f.write_fmt(format_args!("{:?}", or))
+            f.write_fmt(format_args!("{or:?}"))
         } else if let Some(regex_prefix) = &self.regex_prefix {
-            f.write_fmt(format_args!("{:?}", regex_prefix))
+            f.write_fmt(format_args!("{regex_prefix:?}"))
         } else if let Some(compare_prefix) = &self.compare_prefix {
-            f.write_fmt(format_args!("{:?}", compare_prefix))
+            f.write_fmt(format_args!("{compare_prefix:?}"))
         } else {
-            f.write_fmt(format_args!(""))
+            f.write_str("")
         }
     }
 }
 
 #[derive(Debug)]
-pub enum QueryComparison {
+enum QueryComparison {
     Equal,
     Less,
     Greater,
@@ -832,25 +917,25 @@ pub enum QueryComparison {
 }
 
 #[derive(Debug)]
-pub enum StringQuery {
+enum StringQuery {
     Value(String),
-    Regex(regex::Regex),
+    Regex(Regex),
 }
 
 #[derive(Debug)]
-pub enum ComparableQuery {
+enum ComparableQuery {
     Numerical(NumericalQuery),
     Time(TimeQuery),
 }
 
 #[derive(Debug)]
-pub struct NumericalQuery {
-    pub condition: QueryComparison,
-    pub value: f64,
+struct NumericalQuery {
+    condition: QueryComparison,
+    value: f64,
 }
 
 #[derive(Debug)]
-pub struct TimeQuery {
-    pub condition: QueryComparison,
-    pub duration: Duration,
+struct TimeQuery {
+    condition: QueryComparison,
+    duration: Duration,
 }
