@@ -1,24 +1,26 @@
 use std::cmp::min;
 
 use tui::{
-    backend::Backend,
     layout::{Constraint, Direction, Layout, Rect},
     terminal::Frame,
     widgets::Block,
 };
 
 use crate::{
-    app::{data_harvester::cpu::CpuDataType, App},
-    canvas::Painter,
-    components::tui_widget::pipe_gauge::{LabelLimit, PipeGauge},
+    app::App,
+    canvas::{
+        components::pipe_gauge::{LabelLimit, PipeGauge},
+        Painter,
+    },
     constants::*,
+    data_collection::cpu::CpuDataType,
     data_conversion::CpuWidgetData,
 };
 
 impl Painter {
     /// Inspired by htop.
-    pub fn draw_basic_cpu<B: Backend>(
-        &self, f: &mut Frame<'_, B>, app_state: &mut App, draw_loc: Rect, widget_id: u64,
+    pub fn draw_basic_cpu(
+        &self, f: &mut Frame<'_>, app_state: &mut App, draw_loc: Rect, widget_id: u64,
     ) {
         // Skip the first element, it's the "all" element
         if app_state.converted_data.cpu_data.len() > 1 {
@@ -31,8 +33,8 @@ impl Painter {
             // many rows and columns we have in draw_loc (-2 on both sides for border?).
             // I think what we can do is try to fit in as many in one column as possible.
             // If not, then add a new column.
-            // Then, from this, split the row space across ALL columns.  From there, generate
-            // the desired lengths.
+            // Then, from this, split the row space across ALL columns.  From there,
+            // generate the desired lengths.
 
             if app_state.current_widget.widget_id == widget_id {
                 f.render_widget(
@@ -62,7 +64,7 @@ impl Painter {
                         last_entry,
                     } => {
                         let (outer, style) = match data_type {
-                            CpuDataType::Avg => ("AVG".to_string(), self.colours.avg_colour_style),
+                            CpuDataType::Avg => ("AVG".to_string(), self.colours.avg_cpu_colour),
                             CpuDataType::Cpu(index) => (
                                 format!("{index:<3}",),
                                 self.colours.cpu_colour_styles
@@ -78,7 +80,7 @@ impl Painter {
 
                 // Very ugly way to sync the gauge limit across all gauges.
                 let hide_parts = columns
-                    .get(0)
+                    .first()
                     .map(|col| {
                         if col.width >= 12 {
                             LabelLimit::None
@@ -92,7 +94,7 @@ impl Painter {
 
                 let num_entries = cpu_data.len();
                 let mut row_counter = num_entries;
-                for (itx, column) in columns.into_iter().enumerate() {
+                for (itx, column) in columns.iter().enumerate() {
                     if REQUIRED_COLUMNS > itx {
                         let to_divide = REQUIRED_COLUMNS - itx;
                         let num_taken = min(
@@ -106,9 +108,11 @@ impl Painter {
                             .direction(Direction::Vertical)
                             .constraints(vec![Constraint::Length(1); remaining_height])
                             .horizontal_margin(1)
-                            .split(column);
+                            .split(*column);
 
-                        for ((start_label, inner_label, ratio, style), row) in chunk.zip(rows) {
+                        for ((start_label, inner_label, ratio, style), row) in
+                            chunk.zip(rows.iter())
+                        {
                             f.render_widget(
                                 PipeGauge::default()
                                     .gauge_style(style)
@@ -117,7 +121,7 @@ impl Painter {
                                     .start_label(start_label)
                                     .ratio(ratio)
                                     .hide_parts(hide_parts),
-                                row,
+                                *row,
                             );
                         }
                     }
