@@ -1,18 +1,17 @@
 //! This mainly concerns converting collected data into things that the canvas
 //! can actually handle.
 
-use kstring::KString;
+// TODO: Split this up!
 
-use crate::app::data_harvester::memory::MemHarvest;
-use crate::app::{
-    data_farmer::DataCollection,
-    data_harvester::{cpu::CpuDataType, temperature::TemperatureType},
-    AxisScaling,
+use std::borrow::Cow;
+
+use crate::{
+    app::{data_farmer::DataCollection, AxisScaling},
+    canvas::components::time_chart::Point,
+    data_collection::{cpu::CpuDataType, memory::MemHarvest, temperature::TemperatureType},
+    utils::{data_prefixes::*, data_units::DataUnit},
+    widgets::{DiskWidgetData, TempWidgetData},
 };
-use crate::components::tui_widget::time_chart::Point;
-use crate::utils::data_units::DataUnit;
-use crate::utils::gen_util::*;
-use crate::widgets::{DiskWidgetData, TempWidgetData};
 
 #[derive(Debug, Default)]
 pub enum BatteryDuration {
@@ -75,7 +74,8 @@ pub struct ConvertedData {
     pub cache_labels: Option<(String, String)>,
     pub swap_labels: Option<(String, String)>,
 
-    pub mem_data: Vec<Point>, /* TODO: Switch this and all data points over to a better data structure... */
+    pub mem_data: Vec<Point>, /* TODO: Switch this and all data points over to a better data
+                               * structure... */
     #[cfg(not(target_os = "windows"))]
     pub cache_data: Vec<Point>,
     pub swap_data: Vec<Point>,
@@ -97,7 +97,7 @@ pub struct ConvertedData {
 
 impl ConvertedData {
     // TODO: Can probably heavily reduce this step to avoid clones.
-    pub fn ingest_disk_data(&mut self, data: &DataCollection) {
+    pub fn convert_disk_data(&mut self, data: &DataCollection) {
         self.disk_data.clear();
 
         data.disk_harvest
@@ -111,27 +111,27 @@ impl ConvertedData {
                 };
 
                 self.disk_data.push(DiskWidgetData {
-                    name: KString::from_ref(&disk.name),
-                    mount_point: KString::from_ref(&disk.mount_point),
+                    name: Cow::Owned(disk.name.to_string()),
+                    mount_point: Cow::Owned(disk.mount_point.to_string()),
                     free_bytes: disk.free_space,
                     used_bytes: disk.used_space,
                     total_bytes: disk.total_space,
                     summed_total_bytes,
-                    io_read: io_read.into(),
-                    io_write: io_write.into(),
+                    io_read: Cow::Owned(io_read.to_string()),
+                    io_write: Cow::Owned(io_write.to_string()),
                 });
             });
 
         self.disk_data.shrink_to_fit();
     }
 
-    pub fn ingest_temp_data(&mut self, data: &DataCollection, temperature_type: TemperatureType) {
+    pub fn convert_temp_data(&mut self, data: &DataCollection, temperature_type: TemperatureType) {
         self.temp_data.clear();
 
         data.temp_harvest.iter().for_each(|temp_harvest| {
             self.temp_data.push(TempWidgetData {
-                sensor: KString::from_ref(&temp_harvest.name),
-                temperature_value: temp_harvest.temperature.ceil() as u64,
+                sensor: Cow::Owned(temp_harvest.name.to_string()),
+                temperature_value: temp_harvest.temperature.map(|temp| temp.ceil() as u64),
                 temperature_type,
             });
         });
@@ -139,7 +139,7 @@ impl ConvertedData {
         self.temp_data.shrink_to_fit();
     }
 
-    pub fn ingest_cpu_data(&mut self, current_data: &DataCollection) {
+    pub fn convert_cpu_data(&mut self, current_data: &DataCollection) {
         let current_time = current_data.current_instant;
 
         // (Re-)initialize the vector if the lengths don't match...
@@ -170,7 +170,8 @@ impl ConvertedData {
                             data,
                             last_entry,
                         } => {
-                            // A bit faster to just update all the times, so we just clear the vector.
+                            // A bit faster to just update all the times, so we just clear the
+                            // vector.
                             data.clear();
                             *last_entry = *cpu_usage;
                         }
@@ -178,8 +179,8 @@ impl ConvertedData {
             }
         }
 
-        // TODO: [Opt] Can probably avoid data deduplication - store the shift + data + original once.
-        // Now push all the data.
+        // TODO: [Opt] Can probably avoid data deduplication - store the shift + data +
+        // original once. Now push all the data.
         for (itx, mut cpu) in &mut self.cpu_data.iter_mut().skip(1).enumerate() {
             match &mut cpu {
                 CpuWidgetData::All => unreachable!(),
@@ -208,11 +209,11 @@ impl ConvertedData {
     }
 }
 
-pub fn convert_mem_data_points(current_data: &DataCollection) -> Vec<Point> {
+pub fn convert_mem_data_points(data: &DataCollection) -> Vec<Point> {
     let mut result: Vec<Point> = Vec::new();
-    let current_time = current_data.current_instant;
+    let current_time = data.current_instant;
 
-    for (time, data) in &current_data.timed_data_vec {
+    for (time, data) in &data.timed_data_vec {
         if let Some(mem_data) = data.mem_data {
             let time_from_start: f64 =
                 (current_time.duration_since(*time).as_millis() as f64).floor();
@@ -227,11 +228,11 @@ pub fn convert_mem_data_points(current_data: &DataCollection) -> Vec<Point> {
 }
 
 #[cfg(not(target_os = "windows"))]
-pub fn convert_cache_data_points(current_data: &DataCollection) -> Vec<Point> {
+pub fn convert_cache_data_points(data: &DataCollection) -> Vec<Point> {
     let mut result: Vec<Point> = Vec::new();
-    let current_time = current_data.current_instant;
+    let current_time = data.current_instant;
 
-    for (time, data) in &current_data.timed_data_vec {
+    for (time, data) in &data.timed_data_vec {
         if let Some(cache_data) = data.cache_data {
             let time_from_start: f64 =
                 (current_time.duration_since(*time).as_millis() as f64).floor();
@@ -245,11 +246,11 @@ pub fn convert_cache_data_points(current_data: &DataCollection) -> Vec<Point> {
     result
 }
 
-pub fn convert_swap_data_points(current_data: &DataCollection) -> Vec<Point> {
+pub fn convert_swap_data_points(data: &DataCollection) -> Vec<Point> {
     let mut result: Vec<Point> = Vec::new();
-    let current_time = current_data.current_instant;
+    let current_time = data.current_instant;
 
-    for (time, data) in &current_data.timed_data_vec {
+    for (time, data) in &data.timed_data_vec {
         if let Some(swap_data) = data.swap_data {
             let time_from_start: f64 =
                 (current_time.duration_since(*time).as_millis() as f64).floor();
@@ -263,31 +264,29 @@ pub fn convert_swap_data_points(current_data: &DataCollection) -> Vec<Point> {
     result
 }
 
-/// Returns the most appropriate binary prefix unit type (e.g. kibibyte) and denominator for the given amount of bytes.
+/// Returns the most appropriate binary prefix unit type (e.g. kibibyte) and
+/// denominator for the given amount of bytes.
 ///
-/// The expected usage is to divide out the given value with the returned denominator in order to be able to use it
-/// with the returned binary unit (e.g. divide 3000 bytes by 1024 to have a value in KiB).
-fn get_mem_binary_unit_and_denominator(bytes: u64) -> (&'static str, f64) {
-    if bytes < KIBI_LIMIT {
-        // Stick with bytes if under a kibibyte.
-        ("B", 1.0)
-    } else if bytes < MEBI_LIMIT {
-        ("KiB", KIBI_LIMIT_F64)
-    } else if bytes < GIBI_LIMIT {
-        ("MiB", MEBI_LIMIT_F64)
-    } else if bytes < TEBI_LIMIT {
-        ("GiB", GIBI_LIMIT_F64)
-    } else {
-        // Otherwise just use tebibytes, which is probably safe for most use cases.
-        ("TiB", TEBI_LIMIT_F64)
+/// The expected usage is to divide out the given value with the returned
+/// denominator in order to be able to use it with the returned binary unit
+/// (e.g. divide 3000 bytes by 1024 to have a value in KiB).
+#[inline]
+fn get_binary_unit_and_denominator(bytes: u64) -> (&'static str, f64) {
+    match bytes {
+        b if b < KIBI_LIMIT => ("B", 1.0),
+        b if b < MEBI_LIMIT => ("KiB", KIBI_LIMIT_F64),
+        b if b < GIBI_LIMIT => ("MiB", MEBI_LIMIT_F64),
+        b if b < TEBI_LIMIT => ("GiB", GIBI_LIMIT_F64),
+        _ => ("TiB", TEBI_LIMIT_F64),
     }
 }
 
-/// Returns the unit type and denominator for given total amount of memory in kibibytes.
+/// Returns the unit type and denominator for given total amount of memory in
+/// kibibytes.
 pub fn convert_mem_label(harvest: &MemHarvest) -> Option<(String, String)> {
     if harvest.total_bytes > 0 {
         Some((format!("{:3.0}%", harvest.use_percent.unwrap_or(0.0)), {
-            let (unit, denominator) = get_mem_binary_unit_and_denominator(harvest.total_bytes);
+            let (unit, denominator) = get_binary_unit_and_denominator(harvest.total_bytes);
 
             format!(
                 "   {:.1}{}/{:.1}{}",
@@ -302,7 +301,7 @@ pub fn convert_mem_label(harvest: &MemHarvest) -> Option<(String, String)> {
     }
 }
 
-pub fn get_rx_tx_data_points(
+pub fn get_network_points(
     data: &DataCollection, scale_type: &AxisScaling, unit_type: &DataUnit, use_binary_prefix: bool,
 ) -> (Vec<Point>, Vec<Point>) {
     let mut rx: Vec<Point> = Vec::new();
@@ -348,11 +347,11 @@ pub fn get_rx_tx_data_points(
     (rx, tx)
 }
 
-pub fn convert_network_data_points(
+pub fn convert_network_points(
     data: &DataCollection, need_four_points: bool, scale_type: &AxisScaling, unit_type: &DataUnit,
     use_binary_prefix: bool,
 ) -> ConvertedNetworkData {
-    let (rx, tx) = get_rx_tx_data_points(data, scale_type, unit_type, use_binary_prefix);
+    let (rx, tx) = get_network_points(data, scale_type, unit_type, use_binary_prefix);
 
     let unit = match unit_type {
         DataUnit::Byte => "B/s",
@@ -377,7 +376,9 @@ pub fn convert_network_data_points(
     let (rx_converted_result, total_rx_converted_result): ((f64, String), (f64, &'static str)) =
         if use_binary_prefix {
             (
-                get_binary_prefix(rx_data, unit), /* If this isn't obvious why there's two functions, one you can configure the unit, the other is always bytes */
+                get_binary_prefix(rx_data, unit), /* If this isn't obvious why there's two
+                                                   * functions, one you can configure the unit,
+                                                   * the other is always bytes */
                 get_binary_bytes(total_rx_data),
             )
         } else {
@@ -470,8 +471,9 @@ pub fn convert_network_data_points(
     }
 }
 
-/// Returns a string given a value that is converted to the closest binary variant.
-/// If the value is greater than a gibibyte, then it will return a decimal place.
+/// Returns a string given a value that is converted to the closest binary
+/// variant. If the value is greater than a gibibyte, then it will return a
+/// decimal place.
 pub fn binary_byte_string(value: u64) -> String {
     let converted_values = get_binary_bytes(value);
     if value >= GIBI_LIMIT {
@@ -492,8 +494,9 @@ pub fn dec_bytes_per_string(value: u64) -> String {
     }
 }
 
-/// Returns a string given a value that is converted to the closest SI-variant, per second.
-/// If the value is greater than a giga-X, then it will return a decimal place.
+/// Returns a string given a value that is converted to the closest SI-variant,
+/// per second. If the value is greater than a giga-X, then it will return a
+/// decimal place.
 pub fn dec_bytes_per_second_string(value: u64) -> String {
     let converted_values = get_decimal_bytes(value);
     if value >= GIGA_LIMIT {
@@ -546,35 +549,7 @@ pub fn convert_battery_harvest(current_data: &DataCollection) -> Vec<ConvertedBa
 }
 
 #[cfg(feature = "zfs")]
-pub fn convert_arc_labels(
-    current_data: &crate::app::data_farmer::DataCollection,
-) -> Option<(String, String)> {
-    if current_data.arc_harvest.total_bytes > 0 {
-        Some((
-            format!(
-                "{:3.0}%",
-                current_data.arc_harvest.use_percent.unwrap_or(0.0)
-            ),
-            {
-                let (unit, denominator) =
-                    get_mem_binary_unit_and_denominator(current_data.arc_harvest.total_bytes);
-
-                format!(
-                    "   {:.1}{unit}/{:.1}{unit}",
-                    current_data.arc_harvest.used_bytes as f64 / denominator,
-                    (current_data.arc_harvest.total_bytes as f64 / denominator),
-                )
-            },
-        ))
-    } else {
-        None
-    }
-}
-
-#[cfg(feature = "zfs")]
-pub fn convert_arc_data_points(
-    current_data: &crate::app::data_farmer::DataCollection,
-) -> Vec<Point> {
+pub fn convert_arc_data_points(current_data: &DataCollection) -> Vec<Point> {
     let mut result: Vec<Point> = Vec::new();
     let current_time = current_data.current_instant;
 
@@ -602,9 +577,7 @@ pub struct ConvertedGpuData {
 }
 
 #[cfg(feature = "gpu")]
-pub fn convert_gpu_data(
-    current_data: &crate::app::data_farmer::DataCollection,
-) -> Option<Vec<ConvertedGpuData>> {
+pub fn convert_gpu_data(current_data: &DataCollection) -> Option<Vec<ConvertedGpuData>> {
     let current_time = current_data.current_instant;
 
     // convert points
@@ -631,7 +604,7 @@ pub fn convert_gpu_data(
     let results = current_data
         .gpu_harvest
         .iter()
-        .zip(point_vec.into_iter())
+        .zip(point_vec)
         .map(|(gpu, points)| {
             let short_name = {
                 let last_words = gpu.0.split_whitespace().rev().take(2).collect::<Vec<_>>();
@@ -644,8 +617,7 @@ pub fn convert_gpu_data(
                 points,
                 mem_percent: format!("{:3.0}%", gpu.1.use_percent.unwrap_or(0.0)),
                 mem_total: {
-                    let (unit, denominator) =
-                        get_mem_binary_unit_and_denominator(gpu.1.total_bytes);
+                    let (unit, denominator) = get_binary_unit_and_denominator(gpu.1.total_bytes);
 
                     format!(
                         "   {:.1}{unit}/{:.1}{unit}",
